/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Contreras
Contributor(s): Philippe Merle.

====================================================================*/

package org.objectweb.ccm.deploytool;

// Package dependencies
import org.objectweb.util.cmdline.api.CommandLine;

import org.objectweb.util.cmdline.lib.ApplicationBase;
import org.objectweb.util.cmdline.lib.DefaultCommandLine;

/**
 * This is the CCM XML deployment application.
 *
 * @author <a href="mailto:Christophe.Contreras@lifl.fr">Christophe Contreras</a>
 *
 * @version 0.1
 */

public class DeploymentApplication
     extends ApplicationBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the deployer. */
    private DeploymentProcess _deployer;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     *  The default constructor.
     */
    public
    DeploymentApplication()
    {
        this(
            new DefaultCommandLine(
                "ccm_deploy"
                ,"assembly_file.cad"
                ,"Deploy an XML Component Assembly Descriptor"
                ,true
                )
            );
    }

    /**
     * The constructor taking a CommandLine.
     *
     * @param commandLine The command line.
     */    
    public
    DeploymentApplication(CommandLine commandLine)
    {
        // Calls the default ApplicationBase constructor.
        super(commandLine);

        // Creates the associated deployer.
        setDeployer(new DeploymentProcess());
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.api.Identifiable
    //
    // ==================================================================

    /**
     * Obtains its identity.
     *
     * @return Its identity.
     */
    public String
    getIdentity()
    {
        return "ccm_deploy"
             + ' ' + org.objectweb.ccm.util.ReleaseInfo.VERSION_MAJOR
             + '.' + org.objectweb.ccm.util.ReleaseInfo.VERSION_MINOR;
    }

    // ==================================================================
    //
    // Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    /**
     * Executes the main function, i.e.
     * 1 - Initializes the CORBA::ORB,
     * 2 - Calls super.runMain().
     *
     * @param args The command line arguments.
     */
    public void
    runMain(String[] args)
    {
        try
        {
            // Initializes the CORBA::ORB singleton.
            args = org.objectweb.ccm.CORBA.TheORB.initialize(args);
        }
        // If any exception thrown then reports it.
        //
        catch(org.objectweb.util.api.ExceptionWrapper exc)
        {
            report_exception(exc.getException());
        }
        catch(Exception exc)
        {
            report_exception(exc);
        }

        // Calls ApplicationBase.runMain method.
        super.runMain(args);
    }

    /**
     * Starts the application.
     *
     * @param args The command line arguments.
     *
     * @return The status.
     */
    public int
    start(String[] args)
    {
       boolean result = deploy(args[0]);
       return result?0:-1;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Obtains the deployer.
     *
     * @return The deployer.
     */
    public DeploymentProcess
    getDeployer()
    {
        return this._deployer;
    }

    /**
     * Sets the deployer.
     *
     * @param deployer The deployer.
     */
    public void
    setDeployer(DeploymentProcess deployer)
    {
        _deployer = deployer;
        _deployer
            .setConsole(
                getConsole()
                );
    }

    /**
     * Deploys an XML Component Assembly Descriptor.
     *
     * @param assembly_filename The XML Component Assembly Descriptor file name.
     *
     * @return True if the deployment successes, otherwise false.
     */
    public boolean
    deploy(String assembly_filename)
    {
        getConsole()
            .message(
                "Initiating deployment sequence of "
                + assembly_filename
                );

        boolean result
            = getDeployer()
                .deploy(
                    assembly_filename
                    );
        if (!result)
        {
            getConsole()
                .error(
                    "Sequence aborted during deployment of "
                    + assembly_filename
                    );
            return false;
        }

        getConsole()
            .message(
                "Successfully deployed "
                + assembly_filename
                );

        return true;
    }

    // ==================================================================
    //
    // Static public methods.
    //
    // ==================================================================

    /**
     * The main bootstrap method.
     *
     * @param args The command line arguments.
     */
    public static void
    main(String[] args)
    {
        DeploymentApplication application = new DeploymentApplication();
        application.runMain(args);
    }
}
