/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.corba.generator.cif.lib;


import org.objectweb.corba.ast.api.*;

import org.objectweb.corba.generator.java.lib.*;
import org.objectweb.corba.generator.java.ast.api.*;
import org.objectweb.corba.generator.java.ast.lib.*;
import org.objectweb.corba.generator.common.lib.GenerationException;

import java.util.Vector;
import java.util.ArrayList;


/**
 * This class generates CIF implementation.
 * Firstly, a java ast is fed, then classes are generated by the JavaGenerator.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</A>
 */

public class CIF_JavaGenerator
     extends org.objectweb.corba.generator.common.lib.GeneratorBase
  implements org.objectweb.corba.generator.cif.api.CIF_JavaGenerator
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The java repository */
    private Repository repository_;

    /** Utility class to convert types in Java*/
    public org.objectweb.corba.generator.java.api.JavaTranslator translator_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     *
     * @param ast - The Abstract Syntax Tree.
     **/
    public CIF_JavaGenerator(org.objectweb.corba.ast.api.AST ast)
    {
        super(ast);
        // Init internal state
        repository_ = null;
        translator_ = new JavaTranslator();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Map a Home Executor definition into a java Class.
     *
     * @param home_executor - The home executor to map.
     * @param package_name - The package name.
     * @param component_executor_name - The component executor name.
     */
    private void
    mapHomeExecutor( HomeExecutorDecl home_executor,
                     String package_name,
                     String component_executor_name )
    {
        ClassObject clazz = null;
        ConstructorObject ct = null;
        MethodObject method = null;

        clazz = new ClassObjectImpl(home_executor.getName());
        clazz.addComment("Home Executor for " + component_executor_name);
        clazz.setAbstract(true);
        clazz.setModifier(ModifierKindImpl.mk_public);
        clazz.setPackage(package_name);
        clazz.addInheritedObject("org.objectweb.ccm.runtime.cif.lib.HomeBase");
        clazz.addImplementedObject("CIF_" + home_executor.getName());

        // Add the constructor
        ct = new ConstructorObjectImpl();
        clazz.addConstructor(ct);

        // Add the create method
        method = new MethodObjectImpl();
        method.addComment("Create a component executor.");
        method.addComment(" ");
        method.addComment("@return The EnterpriseComponent created executor.");
        method.setName("create");
        method.setReturnType("org.omg.Components.EnterpriseComponent");
        method.getImpl().setMacro("HE_CREATE");
        clazz.addMethod(method);

        repository_.addObject(clazz);
    }

    /**
     * Map a Component Executor definition into a java Class.
     *
     * @param comp_exec - The component executor to map.
     * @param package_name - The package name.
     * @param comp_decl - The component to map.
     * @param segments - All segments contained by the component executor.
     * @param context - The associated context.
     */
    private void
    mapComponentExecutor( ExecutorDecl comp_exec,
                          String package_name,
                          ComponentDecl comp_decl,
                          Declaration[] segments,
                          InterfaceDecl context )
    {
        ClassObject clazz = null;
        ConstructorObject ct = null;
        MethodObject method = null;
        AttributeObject att = null;
        ParameterObject param = null;
        CategoryKind kind = null;

        clazz = new ClassObjectImpl(comp_exec.getName());
        clazz.addComment("Component Executor for "+comp_decl.getName());
        clazz.setAbstract(true);
        clazz.setModifier(ModifierKindImpl.mk_public);
        clazz.setPackage(package_name);
        kind = comp_exec.getCompositionKind();
        if ((kind==CategoryKind.ck_entity) || (kind==CategoryKind.ck_process))
            clazz.addInheritedObject("org.objectweb.ccm.runtime.cif.lib.EntityComponentSegmentBase");
        else
            clazz.addInheritedObject("org.objectweb.ccm.runtime.cif.lib.SessionComponentSegmentBase");

        clazz.addImplementedObject("CIF_" + comp_exec.getName());

        // Add the segmentation table attribute
        att = new AttributeObjectImpl();
        att.addComment("The segmentation table.");
        att.setName("the_segmentation_table_");
        att.setType("java.util.HashMap");
        att.setModifier( ModifierKindImpl.mk_private );
        att.setStatic(true);
        clazz.addAttribute(att);

        // Add the _segment_id_value attribute
        att = new AttributeObjectImpl();
        att.addComment("The segment id.");
        att.setName("_segment_id_value");
        att.setType("int");
        att.setModifier( ModifierKindImpl.mk_public );
        att.setFinal(true);
        att.setStatic(true);
        att.setInitialValue("0");
        clazz.addAttribute(att);

        // Add the context attribute
        att = new AttributeObjectImpl();
        att.addComment(" To refer to the component context.");
        att.setName("the_context_");
        att.setType(translator_.getAbsoluteName(context));
        att.setModifier( ModifierKindImpl.mk_private );
        clazz.addAttribute(att);

        // Add the constructor
        ct = new ConstructorObjectImpl();
        ct.getImpl().setMacro("CE_CONSTRUCTOR");
        ct.getImpl().addContextValue("nb", new Integer(segments.length+1));
        clazz.addConstructor(ct);

        // Add a static statement
        method = new MethodObjectImpl();
        method.setName("StaticStatement");
        method.getImpl().setMacro("CE_STATIC_STATEMENT");
        method.getImpl().addContextValue("comp_facet_list", comp_exec.getMainSegmentFacets(comp_decl));
        method.getImpl().addContextValue("segment_list", segments);
        method.getImpl().addContextValue("component_name", comp_decl.getName());
        clazz.addMethod(method);

        // Add the get_segmentation_table() method
        method = new MethodObjectImpl();
        method.addComment("Obtain the segmentation table.");
        method.addComment(" ");
        method.addComment("@return A HashMap containing <facet_name, segid> associations.");
        method.setName("get_segmentation_table");
        method.setReturnType("java.util.HashMap");
        method.getImpl().setMacro("GET_METHOD");
        method.getImpl().addContextValue("var", "the_segmentation_table_");
        clazz.addMethod(method);

        if ((kind==CategoryKind.ck_entity) || (kind==CategoryKind.ck_process))
        {
            // Add the set_entity_context() method

            // Add the unset_entity_context() method
        }
        else
        {
            // Add the set_session_context() method
            method = new MethodObjectImpl();
            method.addComment("Set the session component context.");
            method.addComment("Must be implemented in subclasses.");
            method.addComment(" ");
            method.addComment("@param context The session component context.");
            method.addComment(" ");
            method.addComment("@throw org.omg.Components.CCMException For any problems.");
            method.setName("set_session_context");
            method.setReturnType("void");
            param = new ParameterObjectImpl();
            param.setName("context");
            param.setType("org.omg.Components.SessionContext");
            method.addParameter(param);
            method.addException("org.omg.Components.CCMException");
            method.getImpl().setMacro("SET_METHOD");
            method.getImpl().addContextValue("var", "the_context_");
            method.getImpl().addContextValue("value", "(" + translator_.getAbsoluteName(context) + ")context");
            clazz.addMethod(method);
        }

        // Add the get_context() method
        method = new MethodObjectImpl();
        method.addComment("Obtain the context.");
        method.addComment(" ");
        method.addComment("@return The associated context.");
        method.setName("get_context");
        method.setReturnType(translator_.getAbsoluteName(context));
        method.getImpl().setMacro("GET_METHOD");
        method.getImpl().addContextValue("var", "the_context_");
        clazz.addMethod(method);

        // Add the get_main_segment() method
        method = new MethodObjectImpl();
        method.addComment("Obtain the main executor segment.");
        method.addComment(" ");
        method.addComment("@return The main segment (with id = 0).");
        method.setName("get_main_segment");
        method.setReturnType("CIF_" + comp_exec.getName());
        method.getImpl().setMacro("GET_METHOD");
        method.getImpl().addContextValue("var", "this");
        clazz.addMethod(method);

        repository_.addObject(clazz);
    }

    /**
     * Map a segment definition into a java Class.
     *
     * @param segment - The segment to map.
     * @param package_name - The package name.
     * @param seg_id - The segment id.
     * @param comp_exec - The component executor to map.
     * @param context - The associated context.
     */
    private void
    mapSegment( SegmentDecl segment,
                String package_name,
                int seg_id,
                ExecutorDecl comp_exec,
                InterfaceDecl context )
    {
        ClassObject clazz = null;
        ConstructorObject ct = null;
        MethodObject method = null;
        AttributeObject att = null;
        ParameterObject param = null;

        clazz = new ClassObjectImpl(segment.getName());
        clazz.addComment("Segment");
        clazz.setAbstract(true);
        clazz.setModifier(ModifierKindImpl.mk_public);
        clazz.setPackage(package_name);
        clazz.addInheritedObject("org.objectweb.corba.util.LocalObjectBase");
        clazz.addImplementedObject("CIF_" + segment.getName());
        clazz.addImplementedObject("org.objectweb.ccm.runtime.cif.api.Segment");

        // Add the _segment_id_value attribute
        att = new AttributeObjectImpl();
        att.addComment("The segment id.");
        att.setName("_segment_id_value");
        att.setType("int");
        att.setModifier( ModifierKindImpl.mk_public );
        att.setFinal(true);
        att.setStatic(true);
        att.setInitialValue(new Integer(seg_id).toString());
        clazz.addAttribute(att);

        // Add the main segment attribute
        att = new AttributeObjectImpl();
        att.addComment("To refer to the main segment.");
        att.setName("main_segment_");
        att.setType("CIF_" + comp_exec.getName());
        att.setModifier( ModifierKindImpl.mk_private );
        clazz.addAttribute(att);

        // Add the constructor
        ct = new ConstructorObjectImpl();
        clazz.addConstructor(ct);

        // Add the set_main_segment() method
        method = new MethodObjectImpl();
        method.addComment("Set the main segment executor (segid = 0).");
        method.addComment(" ");
        method.addComment("@param seg - The main segment.");
        method.setName("set_main_segment");
        method.setReturnType("void");
        param = new ParameterObjectImpl();
        param.setName("seg");
        param.setType("org.objectweb.ccm.runtime.cif.api.ComponentSegment");
        method.addParameter(param);
        method.getImpl().setMacro("SET_METHOD");
        method.getImpl().addContextValue("var", "main_segment_");
        method.getImpl().addContextValue("value", "(CIF_" + comp_exec.getName() + ")seg");
        clazz.addMethod(method);

        // Add the get_context() method
        method = new MethodObjectImpl();
        method.addComment("Obtain the context.");
        method.addComment(" ");
        method.addComment("@return The associated context.");
        method.setName("get_context");
        method.setReturnType(translator_.getAbsoluteName(context));
        method.getImpl().setMacro("GET_METHOD");
        method.getImpl().addContextValue("var", " get_main_segment().get_context()");
        clazz.addMethod(method);

        // Add the get_main_segment() method
        method = new MethodObjectImpl();
        method.addComment("Obtain the main executor segment.");
        method.addComment(" ");
        method.addComment("@return The main segment (with id = 0).");
        method.setName("get_main_segment");
        method.setReturnType("CIF_" + comp_exec.getName());
        method.getImpl().setMacro("GET_METHOD");
        method.getImpl().addContextValue("var", "main_segment_");
        clazz.addMethod(method);

        repository_.addObject(clazz);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Generate java files for the CIF.
     *
     * @param base_dir - The base directory where files will be generated.
     * @param scope - The scope to map.
     **/
    public void
    cif_to_java(String base_dir, Scope scope)
    {
        java.io.File out_dir = null;
        repository_ = new RepositoryImpl();
        Declaration[] compositions = null;
        Vector vect = null;

        // Check and create the output directory
        try{
            out_dir = translator_.mkdir(base_dir);
        }catch(GenerationException ex){
            System.err.println(ex.getMessage());
            return;
        }

        /** Get all compositions contained in the file scope **/
        vect = getAllDeclarations(scope, DeclarationKind.dk_composition);
        compositions = (Declaration[])vect.toArray(new Declaration[0]);

        // For each composition, call mapping
        for (int i=0; i<compositions.length; i++)
        {
            CompositionDecl composition = ((CompositionDecl)compositions[i]);
            ComponentDecl component = null;
            Declaration[] decls = null, segments = null;
            HomeExecutorDecl home_exec = null;
            ExecutorDecl comp_exec = null;
            InterfaceDecl context = null;
            String package_name = null;
            ArrayList facet_list = null;

            /** Get the home executor */
            decls = composition.getContents(true, DeclarationKind.dk_home_executor);
              // A composition has 1 home executor.
            home_exec = (org.objectweb.corba.ast.lib.HomeExecutorDeclImpl)decls[0];
            package_name = translator_.getPackage(home_exec);

            /** Get the component executor */
            decls = home_exec.getContents(true, DeclarationKind.dk_executor);
              // A Home Executor has 1 component executor.
            comp_exec = (org.objectweb.corba.ast.lib.ExecutorDeclImpl)decls[0];

            /** Get the associated context */
            component = home_exec.getHomeType().getManagedComponent();
            context = component.getLocalContextMapping();

            /** Get segments */
            segments = comp_exec.getContents(true, DeclarationKind.dk_segment);

            // Call mapping for home executor
            mapHomeExecutor( home_exec,
                             package_name,
                             comp_exec.getName() );
            // Call mapping for component executor
            mapComponentExecutor( comp_exec,
                                  package_name,
                                  component,
                                  segments,
                                  context );
            // Call mapping for segments
            for (int j=0; j<segments.length; j++)
            {
                mapSegment( (SegmentDecl)segments[j],
                            package_name,
                            j+1, // seg_id starts from 1 (0 is reserved to the component executor)
                            comp_exec,
                            context );
            }
        }

        // Generate Java Files
        org.objectweb.corba.generator.java.lib.JavaGenerator java_gen = new JavaGenerator();
        java.util.Vector v = new java.util.Vector();
        v.add("cif.vm");
        java_gen.initialize("CIF", v);
        java_gen.generate(repository_, base_dir);
    }
}
