/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Demarey Christophe.
Contributor(s): .

====================================================================*/

package org.objectweb.util.launcher;

/**
 * Launcher is used to run classes (call the "main" method).
 * Properties are read from an xml file.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class Launcher
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The Run element to launch. */
    private String run_id_ = null;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    Launcher()
    {
        run_id_ = "default";
    }

    // ==================================================================
    //
    // Private methods.
    //
    // ==================================================================

    /**
     * Set the run element to launch.
     *
     * @param id - The id to set.
     */
    private void
    setRunId(String id)
    {
        run_id_ = id;
    }

    /**
     * Get the run element to launch.
     *
     * @return The run id.
     */
    private String
    getRunId()
    {
        return run_id_;
    }

    /**
     * Add properties to the JVM.
     *
     * @param properties - Properties to set.
     */
    private void
    setProperties(java.util.Properties properties)
    {
        System.getProperties().putAll(properties);
    }

    /**
     * Add urls to class loader.
     *
     * @param urls - The list os url to add.
     * @param loader - The classLoader to initialize.
     */
    private void
    updateClasspath(String[] urls, XBootClassLoader loader)
    {
        java.net.URL url = null;

        for (int i=0; i<urls.length; i++)
        {
            try {
                url = new java.net.URL(urls[i]);
            } catch(java.net.MalformedURLException exc) {
                System.err.println("[Error] : Malformed url " + url + "!");
            }
            loader.addURL( url );
         }
    }

    /**
     * Compute Options for the launcher, i.e. :
     * - get the "--runid" option
     * - delete the last argument : the configuration file
     *
     * @param args - Dynamic options.
     *
     * @return An array containing whithout launcher options.
     */
    private String[]
    computeLauncherOptions(String[] args)
    {
        java.util.Vector options = new java.util.Vector();

        for (int i=0; i<args.length-1; i++)
        {
            if ( (args[i].compareTo("--runid")==0) && (i+1<args.length-1) )
            {
                i++;
                setRunId(args[i]);
            }
            else
            {
                options.add(args[i]);
            }
        }
        return (String[])options.toArray(new String[0]);
    }

    /**
     * Compute Options for the class, i.e. :
     * - add dynamic arguments
     * - retrieve static options
     *
     * @param args - Dynamic options.
     * @param static_args - Static options.
     *
     * @return An array containing static and dynamic options.
     */
    private String[]
    computeOptions(String[] args, String[] static_args)
    {
        java.util.Vector options = new java.util.Vector();

        // Add dynamic options (whithout configuration file)
        for (int i=0; i<args.length; i++)
        {
            options.add(args[i]);
        }

        // Add static options
        for (int i=0; i<static_args.length; i++)
        {
            java.util.StringTokenizer st = new java.util.StringTokenizer(static_args[i]);

            while (st.hasMoreTokens())
            {
                 options.add( st.nextToken() );
            }
        }

        return (String[])options.toArray(new String[0]);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Parse an XML launcher configuration file and launch a run Element.
     *
     * @param configurationFile - The xml file to parse.
     * @param args - Arguments from the command line.
     * @return The status (true if ok, else false).
     *
     */
    public boolean
    run(String configurationFile,
        String[] args)
    {
        boolean ok = true;
        String[] classargs = null;
        XBootClassLoader loader = new XBootClassLoader();
        XMLDecoder decoder = new XMLDecoderImpl();


        // Compute Launcher options
        classargs = computeLauncherOptions(args);

        // Parse the xml configuration file
        ok = decoder.parseFile(configurationFile);
        if (!ok)
        {
            System.err.println("[Launcher] : Problem when parsing " + configurationFile);
            return false;
        }

        // Set the Run configuration
        if (!decoder.setRunId( getRunId() ))
            return false;

        // Init class loader
        updateClasspath( decoder.getUrls(), loader );
        // init the class loader for the current thread
        Thread.currentThread().setContextClassLoader(loader);

        // Init properties
        setProperties( decoder.getProperties() );

        // Compute options
        classargs = computeOptions(classargs, decoder.getArguments());

        // Init class to launch
        Class[] params = { args.getClass() };
        Object[] in_params = { classargs };
        String classname = decoder.getMainClassName();

        try{
            java.lang.reflect.Method method = Class.forName(classname, false, loader).getMethod("main", params);
            method.invoke(null, in_params);
        }catch(Exception e){
            System.err.println("[Launcher] Error : ");
            e.printStackTrace();
            return false;
        }
        return true;
    }

    /**
     * The main bootstrap method.
     *
     * @param args The command line arguments.
     *
     * @return The status.
     */
    public static void
    main(String[] args)
    {
        String configurationFile = null;

        Launcher launcher = new Launcher();

        // Get the last argument
        if (args.length < 1)
        {
            System.err.println("Bad usage !");
            System.err.println("java Launcher [options] [--runid <id>] xml_file");
            System.err.println("where xml_file is the configuration file.");
            System.exit(0);
        }
        configurationFile = args[args.length-1];

        // Parse the xml configuration file
        if ( !launcher.run(configurationFile, args) )
            System.exit(0);
    }
}
